//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#ifndef __RMD160_H__
#define __RMD160_H__

//-----------------------------------------------------------------------------
// Define the 32 bit word type.
//
// This type is named rmd160uint32 instead of just uint32 because uint32 is
// sufficiently generic that it may already be defined for other purposes.
// Because some platforms may have more than one type suitable for uint32,
// the generic uint32 may be defined differently than would be defined here.
// This potential conflict is avoided by using a unique name.
//
// The smallest unsigned type that is at least 32 bits is chosen.
//-----------------------------------------------------------------------------
#if ULONG_MAX>4294967295U
#if UINT_MAX>4294967295U
#if USHRT_MAX>4294967295U
#if UCHAR_MAX>=4294967295U
typedef unsigned char rmd160uint32;
#else
typedef unsigned short rmd160uint32;
#endif /*UCHAR_MAX*/
#else
typedef unsigned int rmd160uint32;
#endif /*USHRT_MAX*/
#else
typedef unsigned long rmd160uint32;
#endif /*UINT_MAX*/
#else
typedef unsigned long rmd160uint32;
#endif

//-----------------------------------------------------------------------------
// Define the data structure.
//-----------------------------------------------------------------------------
struct rmd160_object {
    rmd160uint32		data	[16]	;
    rmd160uint32		state	[5]	;
    rmd160uint32		len	[2]	;
    rmd160uint32 *		wptr		;
    rmd160uint32 *		wend		;
    unsigned int		bpos		;
};
typedef struct rmd160_object *	RMD160		;

//-----------------------------------------------------------------------------
// Define error symbols.
//-----------------------------------------------------------------------------
#define RMD160_OK			0
#define RMD160_ERR_INVALID_ARG		1
#define RMD160_ERR_READ			2

//-----------------------------------------------------------------------------
// Declare functions and arguments.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// function	rmd160_append
//
// purpose	Append byte string data to the RMD160 digest.
//
// arguments	1 (RMD160) reference handle to RMD160 instance.
//		2 (size_t) length of byte string to append.
//		3 (const unsigned char *) pointer to the byte string.
//
// returns	(int) RMD160_OK
//		(int) RMD160_ERR_INVALID_ARG
//-----------------------------------------------------------------------------
int
rmd160_append(
	RMD160			arg_obj
,
	size_t			arg_len
,
	const unsigned char *	arg_data
);

//-----------------------------------------------------------------------------
// function	rmd160_append_fd
//
// purpose	Append the contents of an open POSIX file descriptor
//		to the RMD160 checksum.
//
// arguments	1 (RMD160) reference handle to RMD160 instance.
//		2 (int) file descriptor to read from.
//
// returns	(int) RMD160_OK
//		(int) RMD160_ERR_INVALID_ARG
//
// note		If the calling program reads any data from the file
//		descriptor that needs to be included in the checksum,
//		then it must append them with rmd160_append()
//		before calling rmd160_append_fd().
//-----------------------------------------------------------------------------
int
rmd160_append_fd(
	RMD160		arg_obj
,
	int		arg_fd
);

//-----------------------------------------------------------------------------
// function	rmd160_append_file
//
// purpose	Append the contents of an open FILE to the RMD160
//		checksum.
//
// arguments	1 (RMD160) reference handle to RMD160 instance.
//		2 (FILE *) pointer to open file object.
//
// returns	(int) RMD160_OK
//		(int) RMD160_ERR_INVALID_ARG
//
// note		If the calling program reads any data from the file
//		pointer that needs to be included in the checksum, then
//		it must append them with rmd160_append() before
//		calling rmd160_append_fd().
//-----------------------------------------------------------------------------
int
rmd160_append_file(
	RMD160		arg_obj
,
	FILE *		arg_file
);

//-----------------------------------------------------------------------------
// function	_rmd160_calc (internal use only)
//
// purpose	Perform RIPEMD-160 calculation.
//
// arguments	1 (rmd160uint32 *) 5 words of 32 bit state data.
//		2 (rmd160uint32 *) 16 words of 32 bit new data.
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
_rmd160_calc (
	rmd160uint32 *	state
,
	rmd160uint32 *	data
);

//-----------------------------------------------------------------------------
// function	rmd160_copy
//
// purpose	Copy the state from one RMD160 object to another.
//
// arguments	1 (RMD160) reference handle to target RMD160 instance.
//		2 (RMD160) reference handle to source RMD160 instance.
//
// returns	(RMD160) target RMD160 object, allocated if needed.
//-----------------------------------------------------------------------------
RMD160
rmd160_copy (
	RMD160		target_p
,
	RMD160		source_p
);
#define rmd160_dup(s) rmd160_copy(NULL,(s))
#define rmd160_init(t) rmd160_copy((t),NULL)
#define rmd160_new() rmd160_copy(NULL,NULL)


//-----------------------------------------------------------------------------
// function	rmd160_destroy
//
// purpose	Destroy an RMD160 object instance, freeing its memory.
//
// arguments	1 (RMD160) reference handle to RMD160 instance.
//
// returns	(int) RMD160_OK
//		(int) RMD160_ERR_INVALID_ARG
//-----------------------------------------------------------------------------
int
rmd160_destroy (
	RMD160		ptr
);

//-----------------------------------------------------------------------------
// function	rmd160_sum_base16
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 40 4-bit character encoded values.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(char *) pointer to 40 base16 characters where
//				the checksum result is to be stored.
//
// returns	(char *) pointer to returned checksum result.
//		(char *) NULL if error
//-----------------------------------------------------------------------------
char *
rmd160_sum_base16(
	RMD160		source_p
,
	char *		result_p
);

//-----------------------------------------------------------------------------
// function	rmd160_sum_base32
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 32 5-bit character encoded values.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(char *) pointer to 16 base32 characters where
//				the checksum result is to be stored.
//
// returns	(char *) pointer to returned result
//		(char *) NULL if error
//-----------------------------------------------------------------------------
char *
rmd160_sum_base32(
	RMD160		source_p
,
	char *		result_p
);

//-----------------------------------------------------------------------------
// function	rmd160_sum_bytes
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 20 8-bit bytes.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(unsigned char *) pointer to 20 bytes of 8 bits where
//				the checksum result is to be stored.
//
// returns	(unsigned char *) pointer to returned result
//		(unsigned char *) NULL if error
//-----------------------------------------------------------------------------
unsigned char *
rmd160_sum_bytes(
	RMD160			source_p
,
	unsigned char *		result_p
);

//-----------------------------------------------------------------------------
// function	rmd160_sum_words
//
// purpose	Get the resultant checksum of the RMD160 object as
//		an array of 5 32-bit words.
//
// arguments	(RMD160) reference handle to RMD160 instance.
//		(rmd160uint32 *) pointer to 5 words of 32 bits where
//				the checksum result is to be stored.
//
// returns	(rmd160uint32 *) pointer to returned result
//		(rmd160uint32 *) NULL if error
//-----------------------------------------------------------------------------
rmd160uint32 *
rmd160_sum_words(
	RMD160		source_p
,
	rmd160uint32 *	result_p
);

#endif /* __RMD160_H__ */
