//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	rmd160_append
//
// purpose	Append byte string data to the RMD160 digest.
//
// arguments	1 (RMD160) reference handle to RMD160 instance.
//		2 (size_t) length of byte string to append.
//		3 (const unsigned char *) pointer to the byte string.
//
// returns	(int) RMD160_OK
//		(int) RMD160_ERR_INVALID_ARG
//-----------------------------------------------------------------------------

#define FUNCTION_NAME "rmd160_append"

int
rmd160_append (
	RMD160			arg_obj
,
	size_t			arg_len
,
	const unsigned char *	arg_data
)
{
    size_t		alen	;

    rmd160uint32 *	wend	;
    rmd160uint32 *	wptr	;

    unsigned int	bpos	;


    //-- The RMD160 handle must be provided.
    if ( ! arg_obj ) return RMD160_ERR_INVALID_ARG;

    //-- If length is zero, byte string pointer is irrelevant.
    if ( arg_len == 0 ) return RMD160_OK;
    if ( ! arg_data ) return RMD160_ERR_INVALID_ARG;

    alen = arg_len;
    wend = arg_obj->wend;
    wptr = arg_obj->wptr;
    bpos = arg_obj->bpos;

    //-- If there is an incomplete word in the buffer then finish it.
    if ( bpos ) {
	register rmd160uint32 w;
	w = * wptr;
	if ( bpos == 1 ) {
	    w |= ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) ) << 8;
	    -- alen;
	    ++ bpos;
	}
	if ( bpos == 2 && alen ) {
	    w |= ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) ) << 16;
	    -- alen;
	    ++ bpos;
	}
	if ( bpos == 3 && alen ) {
	    w |= ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) ) << 24;
	    -- alen;
	    ++ bpos;
	}
	* wptr = w;
	if ( ! alen ) {
	    arg_obj->wptr = wptr;
	    arg_obj->bpos = bpos;
	    if ( ( arg_obj->len[0] = arg_len + arg_obj->len[0] ) < arg_len ) {
		arg_obj->len[1] ++;
	    }
	    return RMD160_OK;
	}
	bpos = 0;
	++ wptr;
    }

    //-- Put all whole words into buffer, doing calculations when full.
    for (;;) {
	while ( alen >= 4 && wptr < wend ) {

//-----------------------------------------------------------------------------
// Optimize code for little-endian byte order.  GCC do not seem to notice it.
// The non-optimal code does work for little-endian or big-endian byte order.
//-----------------------------------------------------------------------------
#if defined(__BYTE_ORDER) && defined(__LITTLE_ENDIAN) && __BYTE_ORDER == __LITTLE_ENDIAN
	    * wptr = * (const rmd160uint32 *) arg_data;
#else
	    * wptr =
		  ( (rmd160uint32) ( 0xff & arg_data[ 0 ] ) )       |
		  ( (rmd160uint32) ( 0xff & arg_data[ 1 ] ) ) <<  8 |
		  ( (rmd160uint32) ( 0xff & arg_data[ 2 ] ) ) << 16 |
		  ( (rmd160uint32) ( 0xff & arg_data[ 3 ] ) ) << 24;
#endif

	    ++ wptr;
	    arg_data += 4;
	    alen -= 4;
	}
	if ( wptr < wend ) break;
	_rmd160_calc( arg_obj->state , arg_obj->data );
	wptr = arg_obj->data;
    }

    //-- If a partial word remains, put the part in.
    if ( alen ) {
	rmd160uint32 w;
	w = ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) );
	if ( alen >= 2 ) {
	    w |= ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) ) << 8;
	    if ( alen >= 3 ) {
		w |= ( (rmd160uint32) ( 0xff & * ( arg_data ++ ) ) ) << 16;
	    }
	}
	bpos = alen;
	* wptr = w;
    }

    //-- Put object state back.
    arg_obj->wptr = wptr;
    arg_obj->bpos = bpos;
    if ( ( arg_obj->len[0] = arg_len + arg_obj->len[0] ) < arg_len ) {
	arg_obj->len[1] ++;
    }
    return RMD160_OK;

}

#undef FUNCTION_NAME
