//-----------------------------------------------------------------------------
// Copyright  1999 - Philip Howard
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	rmd160
// version	1.2.0
// homepage	http://phil.ipal.org/freeware/rmd160/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil@ipal.org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------

#include "rmd160_lib.h"

//-----------------------------------------------------------------------------
// function	_rmd160_calc (internal use only)
//
// purpose	Perform RIPEMD-160 calculation.
//
// arguments	1 (rmd160uint32 *) 5 words of 32 bit state data.
//		2 (rmd160uint32 *) 16 words of 32 bit new data.
//
// returns	(int) 0
//-----------------------------------------------------------------------------

#define FUNCTION_NAME "rmd160_calc"

int
_rmd160_calc (
	rmd160uint32 *	state
,
	rmd160uint32 *	data
)
{
    rmd160uint32 x0,x1,x2,x3,x4;
    rmd160uint32 y0,y1,y2,y3,y4;

    //--------------------------------------
    // Make two copies of the current state.
    //--------------------------------------
    x0 = y0 = state[0];
    x1 = y1 = state[1];
    x2 = y2 = state[2];
    x3 = y3 = state[3];
    x4 = y4 = state[4];


//-----------------------------------------------------------------------------
// Define the rotate-left macro.
//-----------------------------------------------------------------------------
#define RL(x,n)		(((x) << (n)) | ((x) >> (32-(n))))

//-----------------------------------------------------------------------------
// Define the five functions.
//-----------------------------------------------------------------------------
#define F1(x,y,z)	((x) ^ (y) ^ (z))
#define F2(x,y,z)	(((x) & (y)) | (~(x) & (z)))
#define F3(x,y,z)	(((x) | ~(y)) ^ (z))
#define F4(x,y,z)	(((x) & (z)) | ((y) & ~(z)))
#define F5(x,y,z)	((x) ^ ((y) | ~(z)))

//-----------------------------------------------------------------------------
// Define the ten transforms.
//-----------------------------------------------------------------------------
#define T1(a,b,c,d,e,x,s) {						\
			(a) += F1((b),(c),(d)) + (x);			\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define T2(a,b,c,d,e,x,s) {						\
			(a) += F2((b),(c),(d)) + (x) + 0x5a827999UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define T3(a,b,c,d,e,x,s) {						\
			(a) += F3((b),(c),(d)) + (x) + 0x6ed9eba1UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define T4(a,b,c,d,e,x,s) {						\
			(a) += F4((b),(c),(d)) + (x) + 0x8f1bbcdcUL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define T5(a,b,c,d,e,x,s) {						\
			(a) += F5((b),(c),(d)) + (x) + 0xa953fd4eUL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}

#define S1(a,b,c,d,e,x,s) {						\
			(a) += F1((b),(c),(d)) + (x);			\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define S2(a,b,c,d,e,x,s) {						\
			(a) += F2((b),(c),(d)) + (x) + 0x7a6d76e9UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define S3(a,b,c,d,e,x,s) {						\
			(a) += F3((b),(c),(d)) + (x) + 0x6d703ef3UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define S4(a,b,c,d,e,x,s) {						\
			(a) += F4((b),(c),(d)) + (x) + 0x5c4dd124UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}
#define S5(a,b,c,d,e,x,s) {						\
			(a) += F5((b),(c),(d)) + (x) + 0x50a28be6UL;	\
			(a) = RL((a),(s)) + (e);			\
			(c) = RL((c),10);}

    //------------------------------------
    // Perform the ten calculation rounds.
    //------------------------------------
    T1( x0 , x1 , x2 , x3 , x4 , data[ 0] , 11 );
    T1( x4 , x0 , x1 , x2 , x3 , data[ 1] , 14 );
    T1( x3 , x4 , x0 , x1 , x2 , data[ 2] , 15 );
    T1( x2 , x3 , x4 , x0 , x1 , data[ 3] , 12 );
    T1( x1 , x2 , x3 , x4 , x0 , data[ 4] ,  5 );
    T1( x0 , x1 , x2 , x3 , x4 , data[ 5] ,  8 );
    T1( x4 , x0 , x1 , x2 , x3 , data[ 6] ,  7 );
    T1( x3 , x4 , x0 , x1 , x2 , data[ 7] ,  9 );
    T1( x2 , x3 , x4 , x0 , x1 , data[ 8] , 11 );
    T1( x1 , x2 , x3 , x4 , x0 , data[ 9] , 13 );
    T1( x0 , x1 , x2 , x3 , x4 , data[10] , 14 );
    T1( x4 , x0 , x1 , x2 , x3 , data[11] , 15 );
    T1( x3 , x4 , x0 , x1 , x2 , data[12] ,  6 );
    T1( x2 , x3 , x4 , x0 , x1 , data[13] ,  7 );
    T1( x1 , x2 , x3 , x4 , x0 , data[14] ,  9 );
    T1( x0 , x1 , x2 , x3 , x4 , data[15] ,  8 );

    T2( x4 , x0 , x1 , x2 , x3 , data[ 7] ,  7 );
    T2( x3 , x4 , x0 , x1 , x2 , data[ 4] ,  6 );
    T2( x2 , x3 , x4 , x0 , x1 , data[13] ,  8 );
    T2( x1 , x2 , x3 , x4 , x0 , data[ 1] , 13 );
    T2( x0 , x1 , x2 , x3 , x4 , data[10] , 11 );
    T2( x4 , x0 , x1 , x2 , x3 , data[ 6] ,  9 );
    T2( x3 , x4 , x0 , x1 , x2 , data[15] ,  7 );
    T2( x2 , x3 , x4 , x0 , x1 , data[ 3] , 15 );
    T2( x1 , x2 , x3 , x4 , x0 , data[12] ,  7 );
    T2( x0 , x1 , x2 , x3 , x4 , data[ 0] , 12 );
    T2( x4 , x0 , x1 , x2 , x3 , data[ 9] , 15 );
    T2( x3 , x4 , x0 , x1 , x2 , data[ 5] ,  9 );
    T2( x2 , x3 , x4 , x0 , x1 , data[ 2] , 11 );
    T2( x1 , x2 , x3 , x4 , x0 , data[14] ,  7 );
    T2( x0 , x1 , x2 , x3 , x4 , data[11] , 13 );
    T2( x4 , x0 , x1 , x2 , x3 , data[ 8] , 12 );

    T3( x3 , x4 , x0 , x1 , x2 , data[ 3] , 11 );
    T3( x2 , x3 , x4 , x0 , x1 , data[10] , 13 );
    T3( x1 , x2 , x3 , x4 , x0 , data[14] ,  6 );
    T3( x0 , x1 , x2 , x3 , x4 , data[ 4] ,  7 );
    T3( x4 , x0 , x1 , x2 , x3 , data[ 9] , 14 );
    T3( x3 , x4 , x0 , x1 , x2 , data[15] ,  9 );
    T3( x2 , x3 , x4 , x0 , x1 , data[ 8] , 13 );
    T3( x1 , x2 , x3 , x4 , x0 , data[ 1] , 15 );
    T3( x0 , x1 , x2 , x3 , x4 , data[ 2] , 14 );
    T3( x4 , x0 , x1 , x2 , x3 , data[ 7] ,  8 );
    T3( x3 , x4 , x0 , x1 , x2 , data[ 0] , 13 );
    T3( x2 , x3 , x4 , x0 , x1 , data[ 6] ,  6 );
    T3( x1 , x2 , x3 , x4 , x0 , data[13] ,  5 );
    T3( x0 , x1 , x2 , x3 , x4 , data[11] , 12 );
    T3( x4 , x0 , x1 , x2 , x3 , data[ 5] ,  7 );
    T3( x3 , x4 , x0 , x1 , x2 , data[12] ,  5 );

    T4( x2 , x3 , x4 , x0 , x1 , data[ 1] , 11 );
    T4( x1 , x2 , x3 , x4 , x0 , data[ 9] , 12 );
    T4( x0 , x1 , x2 , x3 , x4 , data[11] , 14 );
    T4( x4 , x0 , x1 , x2 , x3 , data[10] , 15 );
    T4( x3 , x4 , x0 , x1 , x2 , data[ 0] , 14 );
    T4( x2 , x3 , x4 , x0 , x1 , data[ 8] , 15 );
    T4( x1 , x2 , x3 , x4 , x0 , data[12] ,  9 );
    T4( x0 , x1 , x2 , x3 , x4 , data[ 4] ,  8 );
    T4( x4 , x0 , x1 , x2 , x3 , data[13] ,  9 );
    T4( x3 , x4 , x0 , x1 , x2 , data[ 3] , 14 );
    T4( x2 , x3 , x4 , x0 , x1 , data[ 7] ,  5 );
    T4( x1 , x2 , x3 , x4 , x0 , data[15] ,  6 );
    T4( x0 , x1 , x2 , x3 , x4 , data[14] ,  8 );
    T4( x4 , x0 , x1 , x2 , x3 , data[ 5] ,  6 );
    T4( x3 , x4 , x0 , x1 , x2 , data[ 6] ,  5 );
    T4( x2 , x3 , x4 , x0 , x1 , data[ 2] , 12 );

    T5( x1 , x2 , x3 , x4 , x0 , data[ 4] ,  9 );
    T5( x0 , x1 , x2 , x3 , x4 , data[ 0] , 15 );
    T5( x4 , x0 , x1 , x2 , x3 , data[ 5] ,  5 );
    T5( x3 , x4 , x0 , x1 , x2 , data[ 9] , 11 );
    T5( x2 , x3 , x4 , x0 , x1 , data[ 7] ,  6 );
    T5( x1 , x2 , x3 , x4 , x0 , data[12] ,  8 );
    T5( x0 , x1 , x2 , x3 , x4 , data[ 2] , 13 );
    T5( x4 , x0 , x1 , x2 , x3 , data[10] , 12 );
    T5( x3 , x4 , x0 , x1 , x2 , data[14] ,  5 );
    T5( x2 , x3 , x4 , x0 , x1 , data[ 1] , 12 );
    T5( x1 , x2 , x3 , x4 , x0 , data[ 3] , 13 );
    T5( x0 , x1 , x2 , x3 , x4 , data[ 8] , 14 );
    T5( x4 , x0 , x1 , x2 , x3 , data[11] , 11 );
    T5( x3 , x4 , x0 , x1 , x2 , data[ 6] ,  8 );
    T5( x2 , x3 , x4 , x0 , x1 , data[15] ,  5 );
    T5( x1 , x2 , x3 , x4 , x0 , data[13] ,  6 );

    S5( y0 , y1 , y2 , y3 , y4 , data[ 5] ,  8 );
    S5( y4 , y0 , y1 , y2 , y3 , data[14] ,  9 );
    S5( y3 , y4 , y0 , y1 , y2 , data[ 7] ,  9 );
    S5( y2 , y3 , y4 , y0 , y1 , data[ 0] , 11 );
    S5( y1 , y2 , y3 , y4 , y0 , data[ 9] , 13 );
    S5( y0 , y1 , y2 , y3 , y4 , data[ 2] , 15 );
    S5( y4 , y0 , y1 , y2 , y3 , data[11] , 15 );
    S5( y3 , y4 , y0 , y1 , y2 , data[ 4] ,  5 );
    S5( y2 , y3 , y4 , y0 , y1 , data[13] ,  7 );
    S5( y1 , y2 , y3 , y4 , y0 , data[ 6] ,  7 );
    S5( y0 , y1 , y2 , y3 , y4 , data[15] ,  8 );
    S5( y4 , y0 , y1 , y2 , y3 , data[ 8] , 11 );
    S5( y3 , y4 , y0 , y1 , y2 , data[ 1] , 14 );
    S5( y2 , y3 , y4 , y0 , y1 , data[10] , 14 );
    S5( y1 , y2 , y3 , y4 , y0 , data[ 3] , 12 );
    S5( y0 , y1 , y2 , y3 , y4 , data[12] ,  6 );

    S4( y4 , y0 , y1 , y2 , y3 , data[ 6] ,  9 );
    S4( y3 , y4 , y0 , y1 , y2 , data[11] , 13 );
    S4( y2 , y3 , y4 , y0 , y1 , data[ 3] , 15 );
    S4( y1 , y2 , y3 , y4 , y0 , data[ 7] ,  7 );
    S4( y0 , y1 , y2 , y3 , y4 , data[ 0] , 12 );
    S4( y4 , y0 , y1 , y2 , y3 , data[13] ,  8 );
    S4( y3 , y4 , y0 , y1 , y2 , data[ 5] ,  9 );
    S4( y2 , y3 , y4 , y0 , y1 , data[10] , 11 );
    S4( y1 , y2 , y3 , y4 , y0 , data[14] ,  7 );
    S4( y0 , y1 , y2 , y3 , y4 , data[15] ,  7 );
    S4( y4 , y0 , y1 , y2 , y3 , data[ 8] , 12 );
    S4( y3 , y4 , y0 , y1 , y2 , data[12] ,  7 );
    S4( y2 , y3 , y4 , y0 , y1 , data[ 4] ,  6 );
    S4( y1 , y2 , y3 , y4 , y0 , data[ 9] , 15 );
    S4( y0 , y1 , y2 , y3 , y4 , data[ 1] , 13 );
    S4( y4 , y0 , y1 , y2 , y3 , data[ 2] , 11 );

    S3( y3 , y4 , y0 , y1 , y2 , data[15] ,  9 );
    S3( y2 , y3 , y4 , y0 , y1 , data[ 5] ,  7 );
    S3( y1 , y2 , y3 , y4 , y0 , data[ 1] , 15 );
    S3( y0 , y1 , y2 , y3 , y4 , data[ 3] , 11 );
    S3( y4 , y0 , y1 , y2 , y3 , data[ 7] ,  8 );
    S3( y3 , y4 , y0 , y1 , y2 , data[14] ,  6 );
    S3( y2 , y3 , y4 , y0 , y1 , data[ 6] ,  6 );
    S3( y1 , y2 , y3 , y4 , y0 , data[ 9] , 14 );
    S3( y0 , y1 , y2 , y3 , y4 , data[11] , 12 );
    S3( y4 , y0 , y1 , y2 , y3 , data[ 8] , 13 );
    S3( y3 , y4 , y0 , y1 , y2 , data[12] ,  5 );
    S3( y2 , y3 , y4 , y0 , y1 , data[ 2] , 14 );
    S3( y1 , y2 , y3 , y4 , y0 , data[10] , 13 );
    S3( y0 , y1 , y2 , y3 , y4 , data[ 0] , 13 );
    S3( y4 , y0 , y1 , y2 , y3 , data[ 4] ,  7 );
    S3( y3 , y4 , y0 , y1 , y2 , data[13] ,  5 );

    S2( y2 , y3 , y4 , y0 , y1 , data[ 8] , 15 );
    S2( y1 , y2 , y3 , y4 , y0 , data[ 6] ,  5 );
    S2( y0 , y1 , y2 , y3 , y4 , data[ 4] ,  8 );
    S2( y4 , y0 , y1 , y2 , y3 , data[ 1] , 11 );
    S2( y3 , y4 , y0 , y1 , y2 , data[ 3] , 14 );
    S2( y2 , y3 , y4 , y0 , y1 , data[11] , 14 );
    S2( y1 , y2 , y3 , y4 , y0 , data[15] ,  6 );
    S2( y0 , y1 , y2 , y3 , y4 , data[ 0] , 14 );
    S2( y4 , y0 , y1 , y2 , y3 , data[ 5] ,  6 );
    S2( y3 , y4 , y0 , y1 , y2 , data[12] ,  9 );
    S2( y2 , y3 , y4 , y0 , y1 , data[ 2] , 12 );
    S2( y1 , y2 , y3 , y4 , y0 , data[13] ,  9 );
    S2( y0 , y1 , y2 , y3 , y4 , data[ 9] , 12 );
    S2( y4 , y0 , y1 , y2 , y3 , data[ 7] ,  5 );
    S2( y3 , y4 , y0 , y1 , y2 , data[10] , 15 );
    S2( y2 , y3 , y4 , y0 , y1 , data[14] ,  8 );

    S1( y1 , y2 , y3 , y4 , y0 , data[12] ,  8 );
    S1( y0 , y1 , y2 , y3 , y4 , data[15] ,  5 );
    S1( y4 , y0 , y1 , y2 , y3 , data[10] , 12 );
    S1( y3 , y4 , y0 , y1 , y2 , data[ 4] ,  9 );
    S1( y2 , y3 , y4 , y0 , y1 , data[ 1] , 12 );
    S1( y1 , y2 , y3 , y4 , y0 , data[ 5] ,  5 );
    S1( y0 , y1 , y2 , y3 , y4 , data[ 8] , 14 );
    S1( y4 , y0 , y1 , y2 , y3 , data[ 7] ,  6 );
    S1( y3 , y4 , y0 , y1 , y2 , data[ 6] ,  8 );
    S1( y2 , y3 , y4 , y0 , y1 , data[ 2] , 13 );
    S1( y1 , y2 , y3 , y4 , y0 , data[13] ,  6 );
    S1( y0 , y1 , y2 , y3 , y4 , data[14] ,  5 );
    S1( y4 , y0 , y1 , y2 , y3 , data[ 0] , 15 );
    S1( y3 , y4 , y0 , y1 , y2 , data[ 3] , 13 );
    S1( y2 , y3 , y4 , y0 , y1 , data[ 9] , 11 );
    S1( y1 , y2 , y3 , y4 , y0 , data[11] , 11 );

    //---------------------------------
    // Compute the resultant new state.
    //---------------------------------
    y3 += x2 + state[1];
    state[1] = state[2] + x3 + y4;
    state[2] = state[3] + x4 + y0;
    state[3] = state[4] + x0 + y1;
    state[4] = state[0] + x1 + y2;
    state[0] = y3;


    //--------
    // Return.
    //--------
    return RMD160_OK;
}

#undef FUNCTION_NAME
